@@ -1,11 +1,11 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "jinx" "2.6"
+(define-package "jinx" "20251013.1018"
   "Enchanted Spell Checker."
   '((emacs  "29.1")
     (compat "30"))
   :url "https://github.com/minad/jinx"
-  :commit "b412673dec759131fe0abb346998b55225fbe771"
-  :revdesc "2.6-0-gb412673dec75"
+  :commit "e0ad6ea90e25c363a419eb5dbc378ae82252ed0a"
+  :revdesc "e0ad6ea90e25"
   :keywords '("convenience" "text")
   :authors '(("Daniel Mendler" . "mail@daniel-mendler.de"))
   :maintainers '(("Daniel Mendler" . "mail@daniel-mendler.de")))
