@@ -1,12 +1,12 @@
 ;;; jinx.el --- Enchanted Spell Checker -*- lexical-binding: t -*-
 
-;; Copyright (C) 2023-2026 Free Software Foundation, Inc.
+;; Copyright (C) 2023-2025 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2023
-;; Package-Version: 2.6
-;; Package-Revision: 2.6-0-gb412673dec75
+;; Package-Version: 20251013.1018
+;; Package-Revision: e0ad6ea90e25
 ;; Package-Requires: ((emacs "29.1") (compat "30"))
 ;; URL: https://github.com/minad/jinx
 ;; Keywords: convenience, text
@@ -72,7 +72,6 @@
   "Idle timer delay."
   :type 'float)
 
-;; TODO Replace with a universal variable in Emacs bug#80071
 (defcustom jinx-languages
   (replace-regexp-in-string
    "\\..*\\'" ""
@@ -186,11 +185,9 @@ of a buffer.  Write a custom predicate instead, see `jinx--predicates'."
   "Maximal number of suggestions shown in the context menu."
   :type 'natnum)
 
-;; TODO Replace with a universal variable in Emacs bug#80071
 (defvar-local jinx-local-words ""
   "File-local words, as a string separated by whitespace.")
 
-;; TODO Replace with a universal variable in Emacs bug#80071
 (defvar-local jinx-dir-local-words ""
   "Directory-local words, as a string separated by whitespace.")
 
@@ -593,8 +590,8 @@ If CHECK is non-nil, always check first."
   (timer-set-function jinx--timer nil)
   (dolist (frame (frame-list))
     (dolist (win (window-list frame 'no-miniwindow))
-      (when-let* ((buffer (window-buffer win))
-                  ((buffer-local-value 'jinx-mode buffer)))
+      (when-let ((buffer (window-buffer win))
+                 ((buffer-local-value 'jinx-mode buffer)))
         (with-current-buffer buffer
           (jinx--check-pending (window-start win) (window-end win)))))))
 
@@ -629,27 +626,23 @@ If CHECK is non-nil, always check first."
                (default-directory (file-name-directory
                                    (or (locate-library c-name t)
                                        (error "Jinx: %s not found" c-name))))
-               (command nil)
-               (inhibit-read-only t))
+               (command
+                `(,cc ,@jinx--compile-flags "-o" ,mod-name ,c-name
+                  ,@(split-string-and-unquote
+                     (condition-case nil
+                         (car (process-lines "pkg-config" "--cflags" "--libs" "enchant-2"))
+                       (error "-I/usr/include/enchant-2 -I/usr/local/include/enchant-2 -L/usr/local/lib -lenchant-2"))))))
           (with-current-buffer (get-buffer-create "*jinx module compilation*")
-            (erase-buffer)
-            (compilation-mode)
-            (insert (format "Jinx: Compiling %s\n" mod-name))
-            (setq command
-                  `(,cc ,@jinx--compile-flags "-o" ,mod-name ,c-name
-                        ,@(split-string-and-unquote
-                           (condition-case nil
-                               (car (process-lines "pkg-config" "--cflags" "--libs" "enchant-2"))
-                             (error
-                              (insert "Jinx: pkgconf or pkg-config not found\n")
-                              "-I/usr/include/enchant-2 -I/usr/local/include/enchant-2 -L/usr/local/lib -lenchant-2")))))
-            (insert (string-join command " ") "\n")
-            (if (equal 0 (apply #'call-process (car command) nil (current-buffer) t (cdr command)))
-                (insert (message "Jinx: %s compiled successfully" mod-name))
-              (let ((msg (format "Jinx: Compilation of %s failed" mod-name)))
-                (insert msg)
-                (pop-to-buffer (current-buffer))
-                (error msg))))
+            (let ((inhibit-read-only t))
+              (erase-buffer)
+              (compilation-mode)
+              (insert (string-join command " ") "\n")
+              (if (equal 0 (apply #'call-process (car command) nil (current-buffer) t (cdr command)))
+                  (insert (message "Jinx: %s compiled successfully" mod-name))
+                (let ((msg (format "Jinx: Compilation of %s failed" mod-name)))
+                  (insert msg)
+                  (pop-to-buffer (current-buffer))
+                  (error msg)))))
           (setq mod-file (expand-file-name mod-name))))
       ;; Initialize Mac spell checker to avoid dead lock (gh:minad/jinx#91).
       (when (and (eq window-system 'mac) (fboundp 'mac-do-applescript))
@@ -672,7 +665,7 @@ See `isearch-open-necessary-overlays' and `isearch-open-overlay-temporary'."
     (dolist (ov (overlays-in (pos-bol) (pos-eol)) restore)
       (let ((inv (overlay-get ov 'invisible)))
         (when (and (invisible-p inv) (overlay-get ov 'isearch-open-invisible))
-          (push (if-let* ((fun (overlay-get ov 'isearch-open-invisible-temporary)))
+          (push (if-let ((fun (overlay-get ov 'isearch-open-invisible-temporary)))
                     (progn
                       (funcall fun ov nil)
                       (lambda () (funcall fun ov t)))
@@ -684,9 +677,9 @@ See `isearch-open-necessary-overlays' and `isearch-open-overlay-temporary'."
   "Open overlays which hide the current line.
 See `isearch-open-necessary-overlays' and `isearch-open-overlay-temporary'."
   (dolist (ov (overlays-in (pos-bol) (pos-eol)))
-    (when-let* ((fun (overlay-get ov 'isearch-open-invisible))
-                ((invisible-p (overlay-get ov 'invisible))))
-      (funcall fun ov))))
+    (when-let (fun (overlay-get ov 'isearch-open-invisible))
+      (when (invisible-p (overlay-get ov 'invisible))
+        (funcall fun ov)))))
 
 (defun jinx--correct-highlight (overlay fun)
   "Highlight and show OVERLAY during FUN."
@@ -717,26 +710,26 @@ See `isearch-open-necessary-overlays' and `isearch-open-overlay-temporary'."
   (use-local-map
    (make-composed-keymap (list jinx-correct-map) (current-local-map))))
 
-(defun jinx--add-suggestion (list ht word group &optional suffix)
+(defun jinx--add-suggestion (list ht word group)
   "Add suggestion WORD to LIST and HT.
-The suggestion will get a prefix key.  A GROUP and a SUFFIX property are
-optionally added."
+The word will be associated with GROUP and get a prefix key."
   (unless (gethash word ht)
     (add-text-properties
      0 (length word)
-     `( jinx--group ,group
-        jinx--prefix ,(let ((idx (1+ (hash-table-count ht))))
-                        (cond
-                         ((< idx 10)
-                          (format #("%d " 0 3 (face jinx-key))
-                                  idx))
-                         ((< (- idx 10) (length jinx--select-keys))
-                          (format #("0%c " 0 4 (face jinx-key))
-                                  (aref jinx--select-keys (- idx 10))))))
-        jinx--suffix ,suffix)
+     (list 'jinx--group group
+           'jinx--prefix
+           (let ((idx (1+ (hash-table-count ht))))
+             (cond
+              ((< idx 10)
+               (format #("%d " 0 3 (face jinx-key))
+                       idx))
+              ((< (- idx 10) (length jinx--select-keys))
+               (format #("0%c " 0 4 (face jinx-key))
+                       (aref jinx--select-keys (- idx 10)))))))
      word)
-    (push word (car list))
-    (puthash word t ht)))
+    (push word list)
+    (puthash word t ht))
+  list)
 
 (defun jinx--session-suggestions (word)
   "Retrieve suggestions for WORD from session."
@@ -750,15 +743,15 @@ optionally added."
 (defun jinx--correct-suggestions (word)
   "Retrieve suggestions for WORD from all dictionaries."
   (let ((ht (make-hash-table :test #'equal))
-        (list (cons nil nil)))
+        (list nil))
     (dolist (dict jinx--dicts)
       (let* ((desc (jinx--mod-describe dict))
              (group (format "Suggestions from dictionary ‘%s’ - %s"
                             (car desc) (cdr desc))))
         (dolist (w (jinx--mod-suggest dict word))
-          (jinx--add-suggestion list ht w group))))
+          (setq list (jinx--add-suggestion list ht w group)))))
     (dolist (w (jinx--session-suggestions word))
-      (jinx--add-suggestion list ht w "Suggestions from session"))
+      (setq list (jinx--add-suggestion list ht w "Suggestions from session")))
     (cl-loop
      for (key . fun) in jinx--save-keys
      for actions = (funcall fun nil key word) do
@@ -770,9 +763,12 @@ optionally added."
                            'face 'jinx-save 'rear-nonsticky t)
       for a2 = (format #(" [%s]" 0 5 (face jinx-annotation)) a)
       do (cl-loop
-          for w2 in (list w (downcase w)) do
-          (jinx--add-suggestion list ht (concat k2 w2) "Accept and save" a2))))
-    (nreverse (car list))))
+          for w2 in (delete-consecutive-dups (list w (downcase w))) do
+          (push (propertize (concat k2 w2)
+                            'jinx--group "Accept and save"
+                            'jinx--suffix a2)
+                list))))
+    (nreverse list)))
 
 (defun jinx--correct-affixation (cands)
   "Affixate CANDS during completion."
@@ -781,6 +777,12 @@ optionally added."
                  (or (get-text-property 0 'jinx--prefix cand) "")
                  (or (get-text-property 0 'jinx--suffix cand) ""))))
 
+(defun jinx--correct-annotation (cand)
+  "Annotate CAND during completion."
+  (if-let ((prefix (get-text-property 0 'jinx--prefix cand)))
+      (format #(" (%s)" 0 5 (face jinx-key)) (string-trim prefix))
+    (get-text-property 0 'jinx--suffix cand)))
+
 (defun jinx--group (word transform)
   "Group WORD during completion, TRANSFORM candidate if non-nil."
   (if transform
@@ -817,7 +819,8 @@ Optionally show prompt INFO and insert INITIAL input."
                           (display-sort-function . ,#'identity)
                           (cycle-sort-function . ,#'identity)
                           (group-function . ,#'jinx--group)
-                          (affixation-function . ,#'jinx--correct-affixation)))
+                          (affixation-function . ,#'jinx--correct-affixation)
+                          (annotation-function . ,#'jinx--correct-annotation)))
                        nil nil initial t word)
                       word)))))
            (len (length choice)))
@@ -832,8 +835,8 @@ Optionally show prompt INFO and insert INITIAL input."
 
 (defun jinx--correct-replace (overlay word)
   "Replace OVERLAY with WORD."
-  (when-let* ((start (overlay-start overlay))
-              (end (overlay-end overlay)))
+  (when-let ((start (overlay-start overlay))
+             (end (overlay-end overlay)))
     (undo-boundary)
     (delete-overlay overlay)
     (goto-char end)
@@ -842,19 +845,19 @@ Optionally show prompt INFO and insert INITIAL input."
 
 (defun jinx--correct-menu (&rest _)
   "Return popup mouse menu to correct misspelling."
-  (when-let* ((posn (event-start last-input-event))
-              (pt (posn-point posn))
-              (ov (car (jinx--get-overlays pt pt t))))
+  (when-let ((posn (event-start last-input-event))
+             (pt (posn-point posn))
+             (ov (car (jinx--get-overlays pt pt t))))
     (let ((menu nil)
           (word (buffer-substring-no-properties
                  (overlay-start ov) (overlay-end ov))))
       (dolist (dict jinx--dicts)
-        (when-let* ((desc (jinx--mod-describe dict))
-                    (suggestions (jinx--mod-suggest dict word)))
+        (when-let ((desc (jinx--mod-describe dict))
+                   (suggestions (jinx--mod-suggest dict word)))
           (push `[,(concat "── " (car desc) " ─ " (cdr desc) " ──") :active nil] menu)
           (cl-loop for w in suggestions repeat jinx-menu-suggestions do
                    (push `[,w (jinx--correct-replace ,ov ,w)] menu))))
-      (when-let* ((suggestions (jinx--session-suggestions word)))
+      (when-let ((suggestions (jinx--session-suggestions word)))
         (push ["── Session ──" :active nil] menu)
         (cl-loop for w in suggestions repeat jinx-menu-suggestions do
           (push `[,w (jinx--correct-replace ,ov ,w)] menu)))
@@ -952,7 +955,7 @@ action KEY."
         (jinx--add-local-word 'jinx-dir-local-words word)
         (let ((default-directory
                (or (locate-dominating-file default-directory ".dir-locals.el")
-                   (when-let* ((proj (project-current)))
+                   (when-let (proj (project-current))
                      (declare-function project-root "project")
                      (project-root proj))
                    default-directory)))
@@ -1013,8 +1016,8 @@ misspelled words, but do not open the correction UI."
        (deactivate-mark)
        (push-mark)
        (while-let ((ov (nth idx overlays)))
-         (if-let* (((overlay-buffer ov))
-                   (skip (jinx--correct-overlay ov :info (format " (%d of %d)" (1+ idx) count))))
+         (if-let (((overlay-buffer ov))
+                  (skip (jinx--correct-overlay ov :info (format " (%d of %d)" (1+ idx) count))))
              (setq idx (mod (+ idx skip) count))
            (cl-incf idx)))))))
 
@@ -1028,9 +1031,9 @@ misspelled words, but do not open the correction UI."
             (count (length overlays))
             (idx 0))
        ;; Not using `while-let' is intentional here.
-       (while (when-let* ((ov (nth idx overlays)))
+       (while (when-let ((ov (nth idx overlays)))
                 (if (overlay-buffer ov)
-                    (when-let* ((skip (jinx--correct-overlay ov)))
+                    (when-let ((skip (jinx--correct-overlay ov)))
                       (setq idx (mod (+ idx skip) count)))
                   (cl-incf idx)))))))) ;; Skip deleted overlay
 
@@ -1050,7 +1053,7 @@ Optionally insert INITIAL input in the minibuffer."
                              (jinx--correct-overlay ov :initial initial)
                          (delete-overlay ov)))))
        (forward-to-word skip)
-       (when-let* ((bounds (jinx--bounds-of-word)))
+       (when-let ((bounds (jinx--bounds-of-word)))
          (setf (cons start end) bounds
                initial nil))))))
 
@@ -1064,11 +1067,15 @@ This command dispatches to the following commands:
     is 4, corresponding to \\[universal-argument] pressed once,
     correct all misspelled words.
   - `jinx-correct-word': If prefix ARG is 16, corresponding to
-    \\[universal-argument] pressed twice, correct word before point."
+    \\[universal-argument] pressed twice, correct word before point.
+  - If prefix ARG is 64, corresponding to \\[universal-argument] pressed
+    three times, check the whole buffer, but do not open the correction
+    UI."
   (interactive "*P")
   (pcase arg
     ('nil (if (use-region-p) (jinx-correct-all) (jinx-correct-nearest)))
     ('(16) (jinx-correct-word))
+    ('(64) (jinx-correct-all t))
     (_ (jinx-correct-all))))
 
 (defun jinx-correct-select ()
@@ -1086,10 +1093,9 @@ This command dispatches to the following commands:
     (insert word)
     (exit-minibuffer)))
 
-(defun jinx-next (&optional n)
+(defun jinx-next (n)
   "Go to to Nth next misspelled word."
   (interactive "p" jinx-mode)
-  (unless n (setq n 1))
   (unless (= n 0)
     (if (minibufferp)
         (throw 'jinx--goto n)
@@ -1099,10 +1105,10 @@ This command dispatches to the following commands:
         (goto-char (overlay-end (nth (mod n (length ov)) ov)))
         (jinx--invisible-open-permanently)))))
 
-(defun jinx-previous (&optional n)
+(defun jinx-previous (n)
   "Go to to Nth previous misspelled word."
   (interactive "p" jinx-mode)
-  (jinx-next (- (or n 1))))
+  (jinx-next (- n)))
 
 ;;;###autoload
 (define-minor-mode jinx-mode
@@ -1123,7 +1129,7 @@ This command dispatches to the following commands:
       (hack-local-variables 'ignore-mode))
     (jinx--get-org-language)
     (setq jinx--exclude-regexp
-          (when-let* ((regexps (jinx--mode-list jinx-exclude-regexps)))
+          (when-let ((regexps (jinx--mode-list jinx-exclude-regexps)))
             (mapconcat (lambda (r) (format "\\(?:%s\\)" r))
                        regexps "\\|"))
           jinx--include-faces (jinx--mode-list jinx-include-faces)
